/*******************************************************************************
 * SAT4J: a SATisfiability library for Java Copyright (C) 2004, 2012 Artois University and CNRS
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU Lesser General Public License Version 2.1 or later (the
 * "LGPL"), in which case the provisions of the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of the LGPL, and not to allow others to use your version of
 * this file under the terms of the EPL, indicate your decision by deleting
 * the provisions above and replace them with the notice and other provisions
 * required by the LGPL. If you do not delete the provisions above, a recipient
 * may use your version of this file under the terms of the EPL or the LGPL.
 *
 * Based on the original MiniSat specification from:
 *
 * An extensible SAT solver. Niklas Een and Niklas Sorensson. Proceedings of the
 * Sixth International Conference on Theory and Applications of Satisfiability
 * Testing, LNCS 2919, pp 502-518, 2003.
 *
 * See www.minisat.se for the original solver in C++.
 *
 * Contributors:
 *   CRIL - initial API and implementation
 *******************************************************************************/
package org.sat4j.tools;

import org.sat4j.specs.ISolverService;
import org.sat4j.specs.Lbool;

/**
 * @since 2.2
 */
public class DecisionTracing extends SearchListenerAdapter<ISolverService> {

    /**
	 * 
	 */
    private static final long serialVersionUID = 1L;

    private int counter;

    private final IVisualizationTool positiveVisu;
    private final IVisualizationTool negativeVisu;
    private final IVisualizationTool restartVisu;
    private final IVisualizationTool cleanVisu;

    private int nVar;

    public DecisionTracing(IVisualizationTool positiveVisu,
            IVisualizationTool negativeVisu, IVisualizationTool restartVisu,
            IVisualizationTool cleanVisu) {
        this.positiveVisu = positiveVisu;
        this.negativeVisu = negativeVisu;
        this.restartVisu = restartVisu;
        this.cleanVisu = cleanVisu;

        this.counter = 1;
    }

    @Override
    public void assuming(int p) {
        if (p > 0) {
            this.positiveVisu.addPoint(this.counter, p);
            this.negativeVisu.addInvisiblePoint(this.counter, 0);
        } else {
            this.negativeVisu.addPoint(this.counter, -p);
            this.positiveVisu.addInvisiblePoint(this.counter, 0);
        }
        this.restartVisu.addInvisiblePoint(this.counter, 0);
        this.cleanVisu.addInvisiblePoint(this.counter, 0);
        this.counter++;
    }

    @Override
    public void restarting() {
        this.restartVisu.addPoint(this.counter, this.nVar);
        this.cleanVisu.addPoint(this.counter, 0);
        this.positiveVisu.addInvisiblePoint(this.counter, 0);
        this.negativeVisu.addInvisiblePoint(this.counter, 0);
    }

    @Override
    public void end(Lbool result) {
        this.positiveVisu.end();
        this.negativeVisu.end();
        this.restartVisu.end();
        this.cleanVisu.end();
    }

    @Override
    public void start() {
        this.counter = 1;
    }

    @Override
    public void init(ISolverService solverService) {
        this.nVar = solverService.nVars();
        this.positiveVisu.init();
        this.negativeVisu.init();
        this.restartVisu.init();
        this.cleanVisu.init();
    }

    @Override
    public void cleaning() {
        this.restartVisu.addPoint(this.counter, 0);
        this.cleanVisu.addPoint(this.counter, this.nVar);
        this.positiveVisu.addInvisiblePoint(this.counter, 0);
        this.negativeVisu.addInvisiblePoint(this.counter, 0);
    }

}
